#include <stdio.h>
#include <stdlib.h>
#include "cap.h"
#include "rtxsrc/rtxMemLeakCheck.h"

int main (int argc, char** argv)
{
   alert_ELEM*  pvalue;
   OSCTXT       ctxt;
   OSCTXT*      pctxt = &ctxt;
   int          i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;
   const OSOCTET* pMsgBuf;

   DECLARE_MEMLEAK_DETECTOR;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [-v] [-o <filename>] [-notrace]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose) rtxSetDiag (&ctxt, 1);

   /* Populate structure of generated type */

   pvalue = rtxMemAllocType (pctxt, alert_ELEM);
   
   Init_alert_ELEM (pctxt, pvalue);
   pvalue->identifier.value = (const OSUTF8CHAR*) "43b080713727";

   pvalue->sender.value = (const OSUTF8CHAR*) "hsas@dhs.gov";

   pvalue->sent.value = (const OSUTF8CHAR*) "2003-04-02T14:39:01-05:00";

   pvalue->status = alert_ELEM_status_Actual;

   pvalue->msgType = alert_ELEM_msgType_Alert;

   pvalue->m.scopePresent = 1;
   pvalue->scope = alert_ELEM_scope_Public;

   { alert_ELEM_info* pdata1;
   OSRTDList* plist = &pvalue->info;
   pdata1 = rtxMemAllocType (pctxt, alert_ELEM_info);
   Init_alert_ELEM_info (pctxt, pdata1);

   { alert_ELEM_info_category* pdata2;
   OSRTDList* plist = &pdata1->category;
   pdata2 = rtxMemAllocType (pctxt, alert_ELEM_info_category);
   *pdata2 = alert_ELEM_info_category_Security;
   rtxDListAppend (pctxt, plist, pdata2);
   }

   pdata1->event.value
       = (const OSUTF8CHAR*) "Homeland Security Advisory System Update";

   pdata1->urgency = alert_ELEM_info_urgency_Immediate;

   pdata1->severity = alert_ELEM_info_severity_Severe;

   pdata1->certainty = alert_ELEM_info_certainty_Likely;

   pdata1->m.senderNamePresent = 1;
   pdata1->senderName.value
       = (const OSUTF8CHAR*) "U.S. Government, Department of Homeland Security";

   pdata1->m.headlinePresent = 1;
   pdata1->headline.value
       = (const OSUTF8CHAR*) "Homeland Security Sets Code ORANGE";

   pdata1->m.descriptionPresent = 1;
   pdata1->description.value
       = (const OSUTF8CHAR*) "The Department of Homeland Security has elevated the Homeland Security Advisory System threat level to ORANGE / High in response to intelligence which may indicate a heightened threat of terrorism.";

   pdata1->m.instructionPresent = 1;
   pdata1->instruction.value
       = (const OSUTF8CHAR*) " A High Condition is declared when there is a high risk of terrorist attacks. In addition to the Protective Measures taken in the previous Threat Conditions, Federal departments and agencies should consider agency specific Protective Measures in accordance with their existing plans.";

   pdata1->m.webPresent = 1;
   pdata1->web.value
       = (const OSUTF8CHAR*) "http://www.dhs.gov/dhspublic/display?theme=29";

   { OSXMLSTRING* pdata3;
   OSRTDList* plist = &pdata1->parameter;
   pdata3 = rtxMemAllocTypeZ (pctxt, OSXMLSTRING);
   pdata3->value = (const OSUTF8CHAR*) "HSAS=ORANGE";
   rtxDListAppend (pctxt, plist, pdata3);
   }

   { alert_ELEM_info_resource* pdata3;
   OSRTDList* plist = &pdata1->resource;
   pdata3 = rtxMemAllocType (pctxt, alert_ELEM_info_resource);
   Init_alert_ELEM_info_resource (pctxt, pdata3);
   pdata3->resourceDesc.value = (const OSUTF8CHAR*) "Image file (GIF)";

   pdata3->m.uriPresent = 1;
   pdata3->uri.value
       = (const OSUTF8CHAR*) "http://www.dhs.gov/dhspublic/getAdvisoryImage";

   rtxDListAppend (pctxt, plist, pdata3);
   }

   { alert_ELEM_info_area* pdata3;
   OSRTDList* plist = &pdata1->area;
   pdata3 = rtxMemAllocType (pctxt, alert_ELEM_info_area);
   Init_alert_ELEM_info_area (pctxt, pdata3);
   pdata3->areaDesc.value
       = (const OSUTF8CHAR*) "U.S. nationwide and interests worldwide";

   rtxDListAppend (pctxt, plist, pdata3);
   }

   rtxDListAppend (pctxt, plist, pdata1);
   }

   /* Encode */

   stat = rtXmlSetEncBufPtr (&ctxt, 0, 0);

   if (0 == stat)
      stat = XmlE_alert (&ctxt, pvalue);

   if (0 == stat) {
      pMsgBuf = rtXmlGetEncBufPtr (&ctxt);
      if (trace) {
         printf ("encoded XML message:\n");
         puts ((char*)pMsgBuf);
         printf ("\n");
      }

      /* Write the encoded message out to the output file */

      stat = rtXmlWriteToFile (&ctxt, filename);
   }

   if (0 != stat && trace) {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
   }

   rtxFreeContext (&ctxt);

   return stat;
}
