/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "employee.h"

#define MAXMSGLEN 4096

static void fill_jSmith (OSCTXT* pctxt, PersonnelRecord* pPR);

int main (int argc, char** argv)
{
   PersonnelRecord employee;
   OSCTXT	ctxt;
   OSOCTET	msgbuf[MAXMSGLEN];
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;
   

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [ -v ] [ -o <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
   rtxSetDiag (&ctxt, verbose);

   /* Populate structure of generated type */

   Init_PersonnelRecord (&ctxt, &employee);

   fill_jSmith (&ctxt, &employee);

   /* Encode */

   stat = rtXmlSetEncBufPtr (&ctxt, msgbuf, sizeof(msgbuf));

   if (0 == stat)
      stat = XmlE_personnelRecord (&ctxt, &employee);

   if (0 == stat) {
      if (trace) {
         printf ("encoded XML message:\n");
         puts ((char*)msgbuf);
         printf ("\n");
      }
   }
   else {
      printf ("Encoding failed\n");
      rtxErrPrint (&ctxt);
      return stat;
   }
  
   /* Write the encoded message out to the output file */

  stat = rtXmlWriteToFile (&ctxt, filename);
  if (0 != stat) {
     printf ("Write to file %s failed\n", filename);
     rtxErrPrint (&ctxt);
  } 
   rtxFreeContext (&ctxt);
   return 0;
}

/* Routine to fill the employee record with test data */

static void fill_Name 
(Name* pName, const char* fn, const char* mi, const char* ln);

static void fill_jSmith (OSCTXT* pctxt, PersonnelRecord* pPR)
{
   ChildInformation* pChildInfo;

   fill_Name (&pPR->name, "John", "P", "Smith");

   pPR->title.value = (const OSUTF8CHAR*) "Director";
   pPR->number = 51;
   pPR->dateOfHire.value = (const OSUTF8CHAR*) "19710917";

   fill_Name (&pPR->nameOfSpouse, "Mary", "T", "Smith");

   rtxDListInit (&pPR->children._elemList);

   /* fill first child record */

   pChildInfo = rtxMemAllocType (pctxt, ChildInformation);
   Init_ChildInformation (pctxt, pChildInfo);
   fill_Name (&pChildInfo->name, "Ralph", "T", "Smith");
   pChildInfo->dateOfBirth.value = (const OSUTF8CHAR*) "19571111";
   rtxDListAppend (pctxt, &pPR->children._elemList, pChildInfo);

   /* fill second child record */

   pChildInfo = rtxMemAllocType (pctxt, ChildInformation);
   Init_ChildInformation (pctxt, pChildInfo);
   fill_Name (&pChildInfo->name, "Susan", "B", "Jones");
   pChildInfo->dateOfBirth.value = (const OSUTF8CHAR*) "19590717";
   rtxDListAppend (pctxt, &pPR->children._elemList, pChildInfo);
}

/* Routine to fill a name structure */

static void fill_Name 
(Name* pName, const char* fn, const char* mi, const char* ln)
{
   pName->givenName.value  = (const OSUTF8CHAR*) fn;
   pName->initial.value    = (const OSUTF8CHAR*) mi;
   pName->familyName.value = (const OSUTF8CHAR*) ln;
}

