/* This test driver program reads an encoded record from a file         */
/* and decodes it and displays the results..                            */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxStreamSocket.h"
#include "rtxmlsrc/rtSaxCParser.h"
#include "employee.h"

#define DEFAULT_URL "http://127.0.0.1:9124"

int main (int argc, char** argv)
{
   PersonnelRecord employee;
   OSCTXT	ctxt;
   int		i, numMsgs=2, stat;
   OSBOOL       trace = TRUE, verbose = FALSE;
   int		port = 0, destPort;
   const char*  url = DEFAULT_URL;
   char buf[100];
   OSIPADDR     ipAddr = OSIPADDR_LOCAL, destIP;
   OSRTSOCKET   listenSocket, socket;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-n")) numMsgs = atoi (argv[++i]);
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else if (!strcmp (argv[i], "-url")) {
            url = argv[++i];
         }
         else {
            printf ("usage: reader [ -v ] [ -i <filename>] [ -notrace ]\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -url  bind URL http://host:port\n");
            printf ("   -n  number of messages to consume\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   {
      char* portStr;

      strcpy (buf, url);
      portStr = strstr (buf, "://");
      if (portStr != 0) {
         url = portStr + 3;
      }
      else
         url = buf;

      portStr = strchr (url, ':');
      if (portStr != 0) {
         *portStr = 0;
         portStr++;
         port = atoi (portStr);
      }
   }

   /* Init context structure */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose) rtxSetDiag (&ctxt, TRUE);

   /* One-time socket package initialization (required for Windows only) */

   stat = rtxSocketsInit();
   if (0 != stat) {
      printf ("Socket initialization failed.\n");
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }

   /* Create listener socket */

   if ((stat = rtxSocketCreate (&listenSocket)) != 0) {
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }

   /* Bind the socket */

   if ((stat = rtxSocketStrToAddr (url, &ipAddr)) != 0) {
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }
   if ((stat = rtxSocketBind (listenSocket, ipAddr, port)) != 0) {
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }

   /* Wait for connection */

   if ((stat = rtxSocketListen (listenSocket, 1)) != 0) {
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (trace) printf ("Wait for connection...\n");

   if ((stat = rtxSocketAccept 
                  (listenSocket, &socket, &destIP, &destPort)) != 0) 
   {
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }
   if (trace) {
      char addr[100];

      if (rtxSocketAddrToStr (destIP, addr, sizeof (addr)) != 0)
         strcpy (addr, "<unknown>");
      printf ("Connection is established with %s:%d\n", addr, destPort);
   }

   /* Init PDU data variable */
   Init_PersonnelRecord (&ctxt, &employee);

   /* Create input source object */
   stat = rtxStreamSocketAttach (&ctxt, socket, OSRTSTRMF_INPUT);
   printf ("decoding %d messages..\n", numMsgs);
   
   for (i = 0; i < numMsgs; i++) {
      /* Call compiler generated decode function */
      if (stat == 0)
         stat = XmlD_personnelRecord (&ctxt, &employee);
   
      if (stat == 0) {
         if (trace) {
            printf ("Decode was successful\n");
            printf ("Decoded record:\n");
            Print_PersonnelRecord ("employee", &employee);
         }
      }
      else {
         printf ("decode failed\n");
         rtxErrPrint (&ctxt);
         return -1;
      }
   }
   rtxStreamClose (&ctxt);
   rtxFreeContext (&ctxt);

   return 0;
}

