// This test driver program encodes a data record and writes the 
// encoded result to an output file..

#include <stdio.h>
#include <stdlib.h>
#include "rtxmlsrc/rtXmlCppMsgBuf.h"
#include "po.h"

int main (int argc, char** argv)
{
   int		i, stat;
   const char*  filename = "message.xml";
   OSBOOL       trace = TRUE, verbose = FALSE;
   const OSOCTET* msgptr;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer [ -v ] [ -o <filename>\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Create buffer and control objects

   PurchaseOrderType po;
   OSXMLEncodeBuffer buffer;
   purchaseOrder_CC pdu (buffer, po);

   buffer.setDiag (verbose);

   // Populate structure of generated type

   po.orderDate.setValue("2003-04-01");
   po.m.orderDatePresent = TRUE;

   po.shipTo.name.setValue("Joe Smith");
   po.shipTo.street.setValue("1 Morning Light Rd");
   po.shipTo.city.setValue("Glenmoore");
   po.shipTo.state.setValue("PA");
   po.shipTo.zip = 19343;

   po.billTo.name.setValue("Bill Jones");
   po.billTo.street.setValue("102 Pickering Way, Suite 506");
   po.billTo.city.setValue("Exton");
   po.billTo.state.setValue("PA");
   po.billTo.zip = 19341;

   po.m.commentPresent = TRUE;
   po.comment.setValue("Need this ASAP!");

   Items_item* pItem = new Items_item();
   pItem->productName.setValue("Acme Widget");
   pItem->quantity = 50;
   pItem->USPrice = 200;
   pItem->partNum.setValue("111-AA");
   po.items.append (pItem);

   pItem = new Items_item();
   pItem->productName.setValue("Wizbang Gadget");
   pItem->quantity = 99;
   pItem->USPrice = 777;
   pItem->m.commentPresent = TRUE;
   pItem->comment.setValue("I would like these in the color red");
   pItem->m.shipDatePresent = TRUE;
   pItem->shipDate.setValue("2003-03-26");
   pItem->partNum.setValue("222-BB");
   po.items.append (pItem);

   // Encode

   stat = pdu.encode();

   if (0 == stat) {
      if (trace) {
         msgptr = buffer.getMsgPtr();
         printf ("encoded XML message:\n");
         printf ("%s\n", (const char*)msgptr);
      }
   }
   else {
      printf ("Encoding failed\n");
      buffer.printErrorInfo();
      return stat;
   }

   // Write the encoded message out to the output file

   if (buffer.write (filename) <= 0) {
      printf ("Can't write to %s\n", filename);
      return -1;
   }

   return 0;
}
