/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/OSRTSocketOutputStream.h"
#include "rtxmlsrc/rtXmlCppMsgBuf.h"
#include "employee.h"

#define DEFAULT_URL "http://127.0.0.1:9124"
#define MAX_CONNECT_ATTEMPTS    3

static void fill_jSmith (OSCTXT* pctxt, PersonnelRecord* pPR);

int main (int argc, char** argv)
{
   int		i, numMsgs = 2, stat, port;
   const char*  url = DEFAULT_URL;
   const char*  filename = 0;
   char         buf[100], *protocol, *address;
   OSBOOL       trace = TRUE, verbose = FALSE;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-n")) numMsgs = atoi (argv[++i]);
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else if (!strcmp (argv[i], "-f")) filename = argv[++i];
         else if (!strcmp (argv[i], "-url")) {
            url = argv[++i];
         }
         else {
            printf ("usage: writer [-v] [-url <url>] [-notrace]\n");
            printf ("   -v         verbose mode: print trace info\n");
            printf ("   -n         number of messages to send\n");
            printf ("   -url       bind URL http://host:port\n");
            printf ("   -f <file>  send encoded file <file> to a stream\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   rtxStrcpy (buf, sizeof(buf), url);
   rtxSocketParseURL (buf, &protocol, &address, &port);

   // Create buffer and control objects

   OSRTSocket socket;

   if (trace) printf ("Connecting to %s:%i...", address, port);
   for (i = 0; i < MAX_CONNECT_ATTEMPTS; i++) {
      stat = socket.connect (address, port);
      if (0 != stat) {
         if (i + 1 == MAX_CONNECT_ATTEMPTS) {
            printf ("\nConnection failed, error code is %i\n", stat);
            return stat;
         }
      }
      else break;
   }
   if (trace) printf ("\nConnection established.\n");

   if (filename == 0) {
      PersonnelRecord value;
      OSXMLEncodeStream stream (new OSRTSocketOutputStream (socket));

      if (stream.getStatus ()) {
         stream.printErrorInfo();
         return stream.getStatus ();
      }

      personnelRecord_CC pdu (stream, value);

      if (verbose) 
         rtxSetDiag (pdu.getCtxtPtr(), 1);

      // Populate structure of generated type

      fill_jSmith (pdu.getCtxtPtr(), &value);

      // Encode

      printf ("sending %d messages..\n", numMsgs);

      for (i = 0; i < numMsgs; i++) {
         stat = pdu.encode();

         if (0 == stat) {
            if (trace) {
               printf ("XML message encoded to stream successfully.\n");
            }
         }
         else {
            printf ("Encoding failed\n");
            stream.printErrorInfo();
            return stat;
         }
      }
   }
   else {
      FILE* fp;

      /* Read input file into a memory buffer */

      if ((fp = fopen (filename, "rb")) != 0) {
         long len;
         OSOCTET* msgbuf;

         /* get the length of file */
         
         fseek (fp, 0, SEEK_END);
         len = ftell (fp);
         fseek (fp, 0, SEEK_SET);

         msgbuf = new OSOCTET [len];
         if (fread (msgbuf, 1, len, fp) != (size_t)len) {
            printf ("Error reading %s.\n", filename);
            return -1;
         }
         fclose (fp);

         for (i = 0; i < numMsgs; i++) {
            stat = socket.send (msgbuf, (int)len);
            if (trace) {
               if (stat == 0) {
                  printf ("Send message successful.\n");
               }
               else {
                  OSCTXT ctxt;
                  printf ("Send failed.\n");
                  rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
                  rtxErrPrint (&ctxt);
                  return stat;
               }
            }
         }

         delete [] msgbuf;
      }
      else {
         printf ("Error opening %s for read access\n", filename);
         return -1;
      }
   }

   return 0;
}

// Routine to fill the employee record with test data

static void fill_Name 
(Name* pName, const char* fn, const char* mi, const char* ln);

static void fill_jSmith (OSCTXT* pctxt, PersonnelRecord* pPR)
{
   ChildInformation* pChildInfo;

   fill_Name (&pPR->name, "John", "P", "Smith");

   pPR->title.setValue ("Director");
   pPR->number.value = 51;
   pPR->dateOfHire.setValue ((const OSUTF8CHAR*)"19710917");

   fill_Name (&pPR->nameOfSpouse, "Mary", "T", "Smith");

   // fill first child record

   pChildInfo = new ChildInformation();
   fill_Name (&pChildInfo->name, "Ralph", "T", "Smith");
   pChildInfo->dateOfBirth.setValue ((const OSUTF8CHAR*)"19571111");
   pPR->children.append (pChildInfo);

   // fill second child record

   pChildInfo = new ChildInformation();
   fill_Name (&pChildInfo->name, "Susan", "B", "Jones");
   pChildInfo->dateOfBirth.setValue ((const OSUTF8CHAR*)"19590717");
   pPR->children.append (pChildInfo);
}

// Routine to fill a name structure

static void fill_Name 
(Name* pName, const char* fn, const char* mi, const char* ln)
{
   pName->givenName.setValue (fn);
   pName->initial.setValue (mi);
   pName->familyName.setValue (ln);
}

